@@ -27,7 +27,7 @@ use List::MoreUtils qw(any);
 use Text::Levenshtein qw(distance);
 
 use Lintian::Data;
-use Lintian::Relation;
+use Lintian::Relation qw(:constants);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(strip);
 
@@ -63,6 +63,7 @@ sub run {
     my $inclcdbs = 0;
 
     my ($bdepends_noarch, $bdepends, %build_systems, $uses_autotools_dev_dh);
+    $bdepends = $info->relation('build-depends-all');
     my $seen_dh = 0;
     my $seen_dh_parallel = 0;
     my $seen_python_helper = 0;
@@ -196,15 +197,16 @@ sub run {
             my $targets = $1;
             $needbuilddepends = 1;
             # Can be multiple targets per rule.
-            while ($targets =~ /\boverride_(dh_[^\s]+)/g) {
-                my $dhcommand = $1;
+            while ($targets =~ /\boverride_dh_([^\s]+)/g) {
+                my $cmd = $1;
+                my $dhcommand = "dh_$cmd";
                 $overrides{$dhcommand} = $.;
                 # If maintainer is using wildcards, it's unlikely to be a typo.
                 next if ($dhcommand =~ /%/);
                 next if ($dh_commands_depends->known($dhcommand));
                 # Unknown command, so check for likely misspellings
                 foreach my $x (sort $dh_commands_depends->all) {
-                    if (distance($dhcommand, $x) < 3) {
+                    if ("dh_auto_$cmd" eq $x or distance($dhcommand, $x) < 3) {
                         tag 'typo-in-debhelper-override-target',
                           "override_$dhcommand", '->', "override_$x",
                           "(line $.)";
@@ -224,7 +226,7 @@ sub run {
                   |blends-dev/rules
                  )
               }xsm
-          ) {
+        ) {
             # All of these indirectly use dh.
             $build_systems{'dh'} = 1;
             delete($build_systems{'debhelper'});
@@ -233,7 +235,7 @@ sub run {
               ^include \s+
                  /usr/share/pkg-kde-tools/qt-kde-team/\d+/debian-qt-kde\.mk
               }xsm
-          ) {
+        ) {
             $build_systems{'dhmk'} = 1;
             delete($build_systems{'debhelper'});
         }
@@ -241,7 +243,6 @@ sub run {
     close($rules_fd);
 
     unless ($inclcdbs){
-        my $bdepends = $info->relation('build-depends-all');
         # Okay - d/rules does not include any file in /usr/share/cdbs/
         tag 'unused-build-dependency-on-cdbs' if ($bdepends->implies('cdbs'));
     }
@@ -278,7 +279,17 @@ sub run {
     }
 
     my $compatnan = 0;
+    my $compatvirtual;
     my $compat_file = $droot->child('compat');
+    my $visit = sub {
+        return 0 unless m,^debhelper-compat \(= (\d+)\)$,;
+        $level = $1;
+        $compatvirtual = $level;
+        tag 'debhelper-compat-virtual-relation', $compatvirtual;
+        return 1;
+    };
+    $bdepends->visit($visit, VISIT_PRED_FULL | VISIT_STOP_FIRST_MATCH);
+
     # Check the compat file.  Do this separately from looping over all
     # of the other files since we use the compat value when checking
     # for brace expansion.
@@ -314,7 +325,7 @@ sub run {
             tag 'debhelper-compat-file-is-empty';
         }
     } else {
-        tag 'debhelper-compat-file-is-missing';
+        tag 'debhelper-compat-file-is-missing' unless $compatvirtual;
     }
 
     if (defined($level) and $level !~ m/^\d+$/ and not $compatnan) {
@@ -456,8 +467,10 @@ sub run {
 
     $bdepends_noarch = $info->relation_noarch('build-depends-all');
     $bdepends = $info->relation('build-depends-all');
-    if ($needbuilddepends && !$bdepends->implies('debhelper')) {
-        tag 'package-uses-debhelper-but-lacks-build-depends';
+    if ($needbuilddepends) {
+        tag 'package-uses-debhelper-but-lacks-build-depends'
+          unless $bdepends->implies('debhelper')
+          or $bdepends->implies('debhelper-compat');
     }
     if ($needdhexecbuilddepends && !$bdepends->implies('dh-exec')) {
         tag 'package-uses-dh-exec-but-lacks-build-depends';
@@ -487,7 +500,8 @@ sub run {
     }
 
     $dh_bd_version = $level if not defined($dh_bd_version);
-    unless ($bdepends->implies("debhelper (>= ${dh_bd_version}~)")){
+    unless ($bdepends->implies("debhelper (>= ${dh_bd_version}~)")
+        or $bdepends->implies("debhelper-compat (= ${dh_bd_version})")){
         my $tagname = 'package-needs-versioned-debhelper-build-depends';
         my @extra = ($level);
         $tagname = 'package-lacks-versioned-build-depends-on-debhelper'
@@ -566,7 +580,7 @@ sub _check_dh_exec {
                        ARCH (?: _OS|_CPU|_BITS|_ENDIAN )?
                       |GNU_ (?:CPU|SYSTEM|TYPE)|MULTIARCH
              ) \Z}xsm
-              ) {
+            ) {
                 tag 'dh-exec-subst-unknown-variable', $path, $sv;
             }
         }
@@ -587,7 +601,7 @@ sub _check_dh_exec {
                        ARCH (?: _OS|_CPU|_BITS|_ENDIAN )?
                       |GNU_ (?:CPU|SYSTEM|TYPE)|MULTIARCH
              ) \Z}xsm
-              ) {
+            ) {
                 if (defined($dv)) {
                     $dhe_useless = ($sv eq $dv);
                 } else {
