@@ -213,7 +213,7 @@ sub process_worklist {
     log_msg('Command line used: ' . join(q{ }, @LINTIAN_CMD));
     while (@worklist) {
         my $len = scalar @worklist;
-        my (@work_splice, @completed, %processed);
+        my (@work_splice, @completed, %processed, %errors);
         my ($lintpipe, $lint_stdin, $status_fd, $lint_status_out);
         my $got_alarm = 0;
 
@@ -299,8 +299,10 @@ sub process_worklist {
             my $time_limit
               = $start_time + BACKLOG_PROCESSING_TIME_LIMIT - time();
             my $count = 0;
+            my $signalled_lintian = 0;
             my $sig_handler = sub {
                 my ($signal_name) = @_;
+                $signalled_lintian = 1;
                 $count++;
                 if ($signal_name eq 'ALRM') {
                     $got_alarm = 1 if $got_alarm >= 0;
@@ -345,6 +347,9 @@ sub process_worklist {
                     log_msg("  [lintian] error processing $group_id "
                           . "(time: $runtime)");
                     $processed{$group_id} = 1;
+                    # We ignore errors if we sent lintian a signal to avoid
+                    # *some* false-positives.
+                    $errors{$group_id} = 1 if not $signalled_lintian;
                 } elsif ($line =~ m/^ack-signal (SIG\S+)$/) {
                     my $signal = $1;
                     log_msg(
@@ -394,6 +399,7 @@ sub process_worklist {
         } else {
             log_msg('Lintian finished successfully');
         }
+        log_msg('Updating the lintian log used for reporting');
         my $filter = generate_log_filter($state, \%processed);
         seek($nfd, 0, SEEK_END);
         update_lintian_log($filter, $nfd, $new_lintian_log);
@@ -410,6 +416,32 @@ sub process_worklist {
             $group_data = $state->{'groups'}{$group_id};
             $group_data->{'last-processed-by'} = $LINTIAN_VERSION;
             delete($group_data->{'out-of-date'});
+            # Always clear the error counter after a successful run.
+            delete($group_data->{'processing-errors'});
+            delete($group_data->{'last-error-by'});
+        }
+        for my $group_id (sort(keys(%errors))) {
+            my $group_data;
+            # In theory, they can disappear - in practise, that requires
+            # an external call to (e.g.) dplint reporting-sync-state.
+            next if not exists($state->{'groups'}{$group_id});
+            $group_data = $state->{'groups'}{$group_id};
+            if ($errors{$group_id}) {
+                if (not exists($group_data->{'last-error-by'})
+                    or $group_data->{'last-error-by'} ne $LINTIAN_VERSION) {
+                    # If it is a new lintian version then (re)set the counter
+                    # to 1.  Case also triggers for the very first issue.
+                    $group_data->{'processing-errors'} = 1;
+                } else {
+                    # Repeated error with the same version
+                    ++$group_data->{'processing-errors'};
+                }
+                # Set the "last-error-by" flag so we can clear the
+                # error if there is a new version of lintian.
+                $group_data->{'last-error-by'} = $LINTIAN_VERSION;
+            } else {
+                delete($group_data->{'processing-errors'});
+            }
         }
         save_state_cache($OPT{'state-dir'}, $state);
         last if $exit_code;
@@ -446,7 +478,7 @@ sub update_lintian_log {
                 m/^N: [ ] Processing [ ] (binary|udeb|source) [ ]
                        package [ ] (\S+) [ ] \(version [ ] (\S+), [ ]
                        arch [ ] (\S+)\)[ ]\.\.\./oxsm
-              ) {
+            ) {
                 my ($type, $pkg, $ver, $arch) = ($1,$2, $3, $4);
                 my $k = "$type:$pkg/$ver";
                 $k .= "/$arch" if $type ne 'source';
